<?php
/**
 * Plugin Name:       Ninjalytics Ecommerce Reports Exporter
 * Description:	      Export reporting settings from various reporting plugins for WooCommerce into a single file.
 * Author:            Aspen Grove Studios
 * Author URI:        https://aspengrovestudios.com/
 * Version:           1.0.0
 * Text Domain:       ags-reporting-preset-exporter
 * License:           GNU General Public License version 3 or later
 * License URI:       https://www.gnu.org/licenses/gpl-3.0.en.html
 * GitLab Plugin URI: https://gitlab.com/aspengrovestudios/ags-reporting-preset-exporter/
 */

/*
Ninjalytics Ecommerce Reports Exporter
Copyright (C) 2021  Aspen Grove Studios

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

============

This plugin includes code based on parts of WordPress by Automattic, released
under GPLv2+, licensed under GPLv3 or later (see wp-license.txt in the license directory
for the license and additional credits applicable to WordPress, and the license.txt
file in the license directory for GPLv3 text).
*/

	
class AGSReportingPresetExporter {

	const	VERSION = '1.0.0',
			SUPPORTED_SOURCES = [
				'hm_psrp' => 'hm_psr_report_settings',
				'hm_xoiwcp' => 'hm_xoiwc_report_settings',
				'hm_psr' => 'hm_psr_report_settings',
				'hm_xoiwc' => 'hm_xoiwc_report_settings',
			];
	
	public static 	$pluginName,
					$pluginBaseUrl,
					$pluginDirectory,
					$pluginFile;
	
	public static function setup() {
		self::$pluginName = __('Ninjalytics Reports Exporter', 'ags-reporting-preset-exporter');
		self::$pluginBaseUrl = plugin_dir_url(__FILE__);
		self::$pluginDirectory = __DIR__.'/';
		self::$pluginFile = __FILE__;
		
		// wp-layouts\ags-layouts.php
		add_action('admin_menu', array(__CLASS__, 'registerAdminPages'));
		add_action('admin_enqueue_scripts', array(__CLASS__, 'adminScripts'));
		
		// phpcs:ignore WordPress.Security.NonceVerification -- no CSRF risk due to read-only operation
		if (isset($_POST['ags_rpx_action']) && $_POST['ags_rpx_action'] == 'export' && !empty($_POST['presets']) && is_array($_POST['presets'])) {
			add_action('admin_init', function() {
				// phpcs:ignore WordPress.Security.NonceVerification, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- no CSRF risk due to read-only operation; source/preset IDs are compared against known values in doExport()
				self::doExport($_POST['presets']);
			});
		}
	}
	
	// wp-layouts\ags-layouts.php
	public static function registerAdminPages() {
		/* Admin Pages */
		add_submenu_page('woocommerce', esc_html(self::$pluginName), esc_html__('Ecommerce Reports Exporter', 'ags-reporting-preset-exporter'), 'manage_options', 'ags-reporting-preset-exporter', [__CLASS__, 'adminPage'] );
	}
	
	public static function adminPage() {
		$presets = self::getPresets();
		include(self::$pluginDirectory.'includes/admin/pages/admin.php');
	}
	
	// wp-layouts\ags-layouts.php
	public static function adminScripts() {
		// phpcs:ignore WordPress.Security.NonceVerification, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- read-only operation, comparing against known value
		if ( isset($_GET['page']) && $_GET['page'] == 'ags-reporting-preset-exporter' ) {
			wp_enqueue_style('ags-reporting-preset-exporter', self::$pluginBaseUrl.'css/admin.css', array(), self::VERSION);
			wp_enqueue_script('ags-reporting-preset-exporter', self::$pluginBaseUrl.'js/admin.js', array('jquery'), self::VERSION);
		}
	}
	
	public static function getPresets() {
		return array_column(
			self::prependSourceId(
				array_intersect_key(
					apply_filters('pp_wc_get_schedulable_email_reports', []),
					self::SUPPORTED_SOURCES
				)
			),
		'reports', 'name');
	}
	
	private static function prependSourceId($presets) {
		array_walk($presets, function(&$value, $key) {
			$sourcePresets = [];
			foreach ($value['reports'] as $presetId => $preset) {
				$sourcePresets[$key.' '.$presetId] = $preset;
			}
			$value['reports'] = $sourcePresets;
		});
		return $presets;
	}
	
	private static function doExport($presets) {
		
		$exportData = [];
		
		foreach ( array_intersect_key($presets, self::SUPPORTED_SOURCES) as $source => $presetIds ) {
			
			$sourcePresets = get_option(self::SUPPORTED_SOURCES[$source], []);
			foreach ($presetIds as $presetId) {
				unset($presetData);
				if ($presetId == 'last') {
					if (!isset($sourcePresets[0])) {
						continue;
					}
					$presetData = $sourcePresets[0];
				} else {
					foreach ( (array) $sourcePresets as $preset ) {
						if (isset($preset['key']) && $preset['key'] == $presetId) {
							$presetData = $preset;
							break;
						}
					}
				}
				
				if (isset($presetData)) {
					$presetName = empty($presetData['preset_name']) ? __('Unnamed preset', 'ags-reporting-preset-exporter') : $presetData['preset_name'];
					$presetData = array_diff_key($presetData, [
						'key' => true,
						'preset_name' => true,
						'email_to' => true,
						'_wp_http_referer' => true,
						'hm-psr-nonce' => true
					]);
					
					if (isset($presetData['order_statuses'])) {
						array_walk( $presetData['order_statuses'], [__CLASS__, 'transformOrderStatus'] );
					}
					
					if (isset($presetData['product_cats'])) {
						array_walk( $presetData['product_cats'], [__CLASS__, 'transformTaxonomyTerm'], 'product_cat' );
					}
					
					if (isset($presetData['product_cats'])) {
						array_walk( $presetData['product_cats'], [__CLASS__, 'transformTaxonomyTerm'], 'product_cat' );
					}
					
					if (isset($presetData['customer_role'])) {
						self::transformUserRole($presetData['customer_role']);
					}
					
					$exportData[] = [
						'name' => $presetName,
						'source' => $source,
						'settings' => $presetData
					];
					
				}
				
			}
		}
		
		if ($exportData) {
			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="presets.arp"');
			wp_send_json( $exportData );
		}
	}
	
	private static function transformOrderStatus(&$status) {
		$status = [
			'id' => $status,
			'name' => wc_get_order_status_name($status)
		];
	}
	
	private static function transformTaxonomyTerm(&$term, $taxonomy) {
		$termObject = get_term($term);
		$term = [
			'id' => $term,
			'name' => empty($termObject->name) ? '#'.$term : $termObject->name
		];
	}
	
	private static function transformUserRole(&$role) {
		$allRoles = wp_roles()->roles;
		$role = [
			'id' => $role,
			'name' => isset($allRoles[$role]) ? translate_user_role($allRoles[$role]['name']) : $role
		];
	}
	
}

AGSReportingPresetExporter::setup();